//
// Created by xiangning zhu on 2021/1/11.
//

#ifndef SAMI_CORE_DEF_H
#define SAMI_CORE_DEF_H

#ifdef __cplusplus
#define SAMI_CORE_EXTERN_C_BEGIN extern "C" {
#define SAMI_CORE_EXTERN_C_END   }
#else
#define SAMI_CORE_EXTERN_C_BEGIN
#define SAMI_CORE_EXTERN_C_END
#endif

#ifdef _WIN32
#ifdef SAMI_CORE_WIN_EXPORT
#define SAMI_CORE_DLL_EXPORT __declspec(dllexport)
#else
#define SAMI_CORE_DLL_EXPORT __declspec(dllimport)
#endif
#else
#define SAMI_CORE_DLL_EXPORT __attribute__((visibility("default")))
#endif
SAMI_CORE_EXTERN_C_BEGIN

#if __cplusplus >= 201103L
#define SAMI_CORE_NON_EXCEPTION noexcept
#else
#define SAMI_CORE_NON_EXCEPTION
#endif  // __cplusplus >= 201103L

#include "sami_core_asr_def.h"
#include "sami_core_auth_check.h"
#include "sami_core_base_def.h"
#include "sami_core_error_code.h"
#include "sami_core_voice_filter_def.h"
#include "sami_core_yggdrasil_def.h"
#include "sami_core_mdss_def.h"

/*************Base****************/
typedef enum SAMI_CORE_DLL_EXPORT logLevel {
    Debug = 4,
    Error = 16,
    Warning = 32,
    Info = 128,
} SAMICoreLogLevel;

typedef enum SAMI_CORE_DLL_EXPORT binaryType { binaryTypeFile, binaryTypeData } SAMICoreBinaryType;

typedef struct SAMI_CORE_DLL_EXPORT property {
    SAMICoreDataType type;
    SAMICorePropertyId id;
    int writable;
    void* data;
    unsigned int dataLen;
} SAMICoreProperty;

typedef struct SAMI_CORE_DLL_EXPORT properties {
    SAMICoreProperty* data;
    unsigned int dataLen;
} SAMICorePropertySet;

typedef struct audioBuffer SAMICoreAudioBuffer;

typedef struct SAMICoreExtendAudioBuffer {
    SAMICoreAudioBuffer samiCoreAudioBuffer;
    bool isLastFrame;
} SAMICoreExtendAudioBuffer;

/*****************************/

/***********Extractor*************/
// use for creating extractor
typedef struct SAMI_CORE_DLL_EXPORT extractorCreateParameter {
    unsigned int sampleRate;
    unsigned int numChannel;
} SAMICoreExtractorCreateParam;

//
typedef struct SAMI_CORE_DLL_EXPORT extractorParameter {
    char* name;
    char* strValue;
    float floatValue;
} SAMICoreExtractorParam;
/*****************************/

/***********Processor*************/
typedef struct SAMI_CORE_DLL_EXPORT audioProcessorParameter {
    int index;
    char name[128];
    float plainValue;
    float minPlainValue;
    float maxPlainValue;
    float scalingPower;
    int type;
    int scalingMode;
} SAMICoreProcessorParameter;
/*****************************/

/***********Processor*************/
typedef struct SAMI_CORE_DLL_EXPORT audioProcessorPrepareParameter {
    unsigned int sampleRate;
    unsigned int blockSize;
} SAMICoreProcessorPrepareParameter;
/*****************************/

typedef struct feature {
    float time;
    float duration;
    float* values;
    unsigned int numValues;
} SAMICoreFeature;

typedef struct featureArray {
    SAMICoreFeature* array;
    unsigned int numFeatures;
    unsigned int featureID;
} SAMICoreFeatureArray;

typedef struct featureSet {
    int numFeatureTypes;
    SAMICoreFeatureArray* set;
} SAMICoreFeatureSet;

/***********3A&Audio processor*************/
typedef struct SAMI_CORE_DLL_EXPORT audio3ACreateParameter {
    unsigned int sampleRate;
    unsigned int channels;
} SAMICore3ACreateParameter;

typedef struct audio3ACreateParameter SAMICoreAudioCommonParameter;

typedef struct SAMI_CORE_DLL_EXPORT audio3ASeekParameter {
    unsigned int pos;
    unsigned int mod;
} SAMICore3ASeekParameter;

typedef struct aecInput {
    SAMICoreAudioBuffer* ref;
    SAMICoreAudioBuffer* mic;
} SAMICoreAecInput;

typedef struct loudnormProperty {
    float source_lufs;
    float source_peak;
    float target_lufs;
} SAMICoreLoudnormProperty;

typedef struct loudnorm2Property {
    float target_lufs;
} SAMICoreLoudnorm2Property;

typedef struct cherEffectProperty {
    float major;
    float seek_position;
} SAMICoreCherEffectProperty;

typedef struct AudioEffectCompressorProperty {
    float pre_gain;
    float threshold;
    float knee;
    float ratio;
    float attack;
    float release;
    float pre_delay;
    float release_zone_1;
    float release_zone_2;
    float release_zone_3;
    float release_zone_4;
    float post_gain;
    float wet;
    float attenuation_dB_thd;
    float detector_avg_thd;
} SAMICoreAudioEffectCompressorProperty;

typedef struct audioFadingProperty {
    float content_duration;
    float fade_in_duration;
    float fade_out_duration;
    float curve_fadein;
    float curve_fadeout;
} SAMICoreAudioFadingProperty;

/*****************************/

/*********time scaler**********/
typedef struct SAMI_CORE_DLL_EXPORT audioTimeScalerCreateParameter {
    unsigned int sampleRate;
    unsigned int channels;
} SAMICoreTimeScalerCreateParameter;
/*****************************/

/*********pitch shifter**********/
typedef struct SAMI_CORE_DLL_EXPORT audioPitchShifterCreateParameter {
    unsigned int sampleRate;
    unsigned int channels;
} SAMICorePitchShifterCreateParameter;
/*****************************/

/*********audio scratcher**********/
typedef struct SAMI_CORE_DLL_EXPORT audioScratcherCreateParameter {
    unsigned int sampleRate;
    unsigned int channels;
} SAMICoreAudioScratcherCreateParameter;
/*****************************/

/*********audio retarget**********/
/**
 * this struct use to set retarget inout params
 * @param in_audio_path  input audio file path;
 * @param out_audio_path output audio file path;
 * @param loop_info_path input file beatinfo json file path;
 * @param target_sec     target total time_sec when process finished
 * @param range_start_sec  start time in input audiofile
 * @param range_end_sec    end time in input audiofile
 * @param process_mode      run mode;@see AudioRetargetProcessMode
 * @param splice_info_jsonstr
 * @param loop_num_limit
 */
typedef enum SAMI_CORE_DLL_EXPORT audioRetargetProcessMode {
    audioRetaretModeAudioInOut = 0,    /*< input audiofile ,output retargeted audiofile*/
    audioRetaretModeGetSplice = 1,     /*< input audiofile ,output retargted splice json strings*/
    audioRetaretModeSpliceAudioOut = 2 /* input audiofile and splice json str ,output audio file*/
} AudioRetargetProcessMode;
typedef struct SAMI_CORE_DLL_EXPORT audioRetargetInOutInfoParameter {
    const char* in_audio_path = nullptr;
    const char* out_audio_path = nullptr;
    const char* loop_info_path = nullptr;
    float target_sec = -1;
    float range_start_sec = 0;
    float range_end_sec = -1;
    AudioRetargetProcessMode process_mode=audioRetaretModeAudioInOut;
    const char* splice_info_jsonstr = nullptr;
    int loop_num_limit;
} SAMICoreAudioRetargetInOutInfoParameter;

typedef void (*SAMICoreCallbackMethod_AudioRetargetOnProgress)(float progress, void* pUser);
typedef void (*SAMICoreCallbackMethod_AudioRetargetOnFinish)(bool success, void* pUser);
typedef void (*SAMICoreCallbackMethod_AudioRetargetOnCancelled)(void* pUser);
/**
 * this struct use to set retarget process callbacks
 */
typedef struct SAMI_CORE_DLL_EXPORT audioRetargetCallBackParameter {
    SAMICoreCallbackMethod_AudioRetargetOnProgress onProgress;
    SAMICoreCallbackMethod_AudioRetargetOnFinish onFinish;
    SAMICoreCallbackMethod_AudioRetargetOnCancelled onCancelled;
    void* userParam;
} SAMICoreAudioRetargetCallBackParameter;

/*****************************/

/*********time align**********/
typedef SAMICoreAecInput SAMICoreTimeAlignParameter;
/*****************************/
/**
 * this struct used for creating context handle, @see SAMICoreIdentify_Processor_ContextWithString @see
 SAMICoreIdentify_Processor_ContextWithFileName
 * here is a small example:
 *
 * @code
 * const char* json_path = "/path/to/json/a.json";
 * SAMICoreContextCreateParameter create_param;
 * create_param.input = json;
 * create_param.resource_paths = NULL;
 * create_param.num_paths = 0;
 *
 * int ret = SAMICoreCreateHandleByIdentify(
 *              &handle, SAMICoreIdentify::SAMICoreIdentify_Processor_ContextWithFileName,
 *              &create_param);
 *
 * @endcode
 *
 * if there are resource paths that needs oto set, then you can write link this:
 * @code
 * SAMICoreContextCreateParameter create_param;
 * const char* json_path = "/path/to/json/a.json";
 * const char* path0 = {"./"};
 * const char* path1 = {"../"};
 * vector<const char*> paths{path0, path1};
 *
 * create_param.resource_paths = paths.data();
 * create_param.num_paths = 2;
 *
 * int ret = SAMICoreCreateHandleByIdentify(
                &handle, SAMICoreIdentify::SAMICoreIdentify_Processor_ContextWithFileName,
                (void *)(&create_param));
 *
 * @endcode
 */
typedef struct SAMI_CORE_DLL_EXPORT resourceParameter {
    /**
     * input json file path or json string, depends on which identity used
     *  1. SAMICoreIdentify_Processor_ContextWithStringWithResourcePaths, 'input' must be json string
     *  2. SAMICoreIdentify_Processor_ContextWithFileNameWithResourcePaths, 'input' must be json file path
     */
    const char* input;

    /**
     * number of resource search paths, can be 0 if there is no resource search paths
     */
    int numPaths;

    /**
     * resource search paths, is a string array.
     * the context use these paths to find resources it needed
     */
    const char** resourcePaths;
} SAMICoreResourceParameter;

/*********engine**********/
typedef struct SAMI_CORE_DLL_EXPORT engineCreateParameter {
    const char* resourcePath;
    int sampleRate;
} SAMICoreEngineCreateParameter;

typedef enum SAMI_CORE_DLL_EXPORT executorTaskType {
    TIME_ALIGN_MODEL = 0,
} SAMICoreExecutorTaskType;

typedef struct SAMI_CORE_DLL_EXPORT executorCreateParameter {
    void* engineHandle;
    int sampleRate;
    SAMICoreExecutorTaskType taskType;
    int outputNum;
} SAMICoreExecutorCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT executorContextCreateParameter {
    int sampleRate;
    int maxBlockSize;
    const char* modelBuffer;
    int modelLen;
    int numChannel;
    const char* bussinessInfo;
} SAMICoreExecutorContextCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT executorContextCreateKWSParameter {
    int sampleRate;
    int maxBlockSize;
    const char* language;
    const char* modelBuffer;
    int modelLen;
    int numChannel;
} SAMICoreExecutorContextCreateKWSParameter;

typedef struct SAMI_CORE_DLL_EXPORT binaryContextCreateParameter {
    SAMICoreBinaryType type;
    void* data;
    int dataLen;
} SAMICoreBinaryContextCreateParameter;

typedef SAMICoreBinaryContextCreateParameter SAMICoreBinaryContextUpdateParameter;

typedef struct SAMI_CORE_DLL_EXPORT parameterEvent {
    int portIndex;
    int parameterIndex;
    float normalizedValue;
    float plainValue;
    const char* processorName;
} SAMICoreContextParameterEvent;

typedef struct SAMI_CORE_DLL_EXPORT jsonContextCreateParameter {
    int sampleRate;
    int maxBlockSize;
    const char* modelBuffer;
    int modelLen;
    int numChannel;
    const char* json_string;
} SAMICoreJSONContextCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT tuneToTargetCreateParameter {
    int sampleRate;
    const char* midiPath;
    float shift;
    float lag;
} SAMICoreTuneToTargetCreateParameter;

typedef struct wakeupWordResult {
    char* wakeupWorld;
    float score;
    int startTime;
    int stopTime;
} SAMICoreWakeupWordResult;

/***************** Server API ******************************/

typedef struct speechDisfluencyCreateParameter {
    SAMICoreServerCommonParameter commonParameter;
} SAMICoreSpeechDisfluencyContextCreateParameter;

typedef struct speechDisfluencyResult {
    char* result;
} SAMICoreSpeechDisfluencyResult;

typedef struct ttsContextCreateParameter {
    const char* url;
    const char* appKey;
    SAMICoreTokenType tokenType;
    const char* token;
    const char* speaker;
    const char* format;
    const char* textType;
    const char* text;
    bool enableConcurrency;
    long timestamp;
    int sampleRate;
    int speechRate;
    int pitchRate;
    bool enableTimestamp;
    int silenceDuration;
    bool enableNetTransportCompress;
    int bitRate;
    bool enableSaveAudioToFile;
    const char* saveFileDir;
    SAMICoreServerCallbackParameter* callbackParameter;
    bool enable_split;
} SAMICoreTtsContextCreateParameter;

typedef struct ttsProcessParameter {
    const char* speaker;
    const char* textType;
    const char* text;
    bool enableConcurrency;
    int sampleRate;
    long timestamp;
    bool enablePlaying;
    bool enableSaveAudioToFile;
} SAMICoreTtsProcessParameter;

typedef struct ttsResult {
    float duration;
    char* phonemeAlignment;
    char* wordAlignment;
    char* data;
    int dataLength;
} SAMICoreTtsResult;

typedef struct LyricsAlignmentContextCreateParameter {
    const char* url;
    const char* appKey;
    const char* token;
    const char* lrcLang;
} SAMICoreLyricsAlignmentContextCreateParameter;

typedef struct LyricsAlignmentResult {
    char* krc_result;
    int len;
} SAMICoreLyricsAlignmentResult;

typedef struct beatTrackingContextCreateParameter {
    SAMICoreServerCommonParameter commonParameter;
    bool groupResult;
} SAMICoreBTContextCreateParameter;

typedef struct beatTrackingResult {
    char* result;
} SAMICoreBeatTrackingResult;

typedef struct musicSourceSeparateContextParameter {
    SAMICoreServerCommonParameter commonParameter;
    const char* model;  // vocal/acc
    const char* outputFormat;
    int outputSampleRate;
    int outputChannel;
} SAMICoreMusicSourceSeparateContextParameter;

typedef struct musicSourceSeparateResult {
    char* data;
    int dataLength;
} SAMICoreMusicSourceSeparateResult;

typedef struct musicTaggingContextParameter {
    SAMICoreServerCommonParameter commonParameter;
    bool enableTag;
    int topNum;
} SAMICoreMusicTaggingContextParameter;

typedef struct musicTaggingResult {
    char* result;
} SAMICoreMusicTaggingResult;

typedef struct chorusContextParameter {
    SAMICoreServerCommonParameter commonParameter;
} SAMICoreChorusContextParameter;

typedef struct chorusResult {
    char* result;
} SAMICoreChorusResult;

typedef struct musicRetargetContextParameter {
    SAMICoreServerCommonParameter commonParameter;
    bool stamp;
    int targetSecs;
} SAMICoreMusicRetargetContextParameter;

typedef struct musicRetargetResult {
    char* result;
} SAMICoreMusicRetargetResult;

typedef struct midiContextParameter {
    SAMICoreServerCommonParameter commonParameter;
    bool midi;
} SAMICoreMidiContextParameter;

typedef struct beamformingCreateParameter {
    char* mic_positions;
    int numMic;
    int sampleRate;
    int chunkSizeMs;
} SAMICoreBeamformingCreateParameter;

typedef struct midiResult {
    char* jsonResult;
    char* data;
    int dataLength;
} SAMICoreMidiResult;

struct SAMICoreServerEvent {
    char* event;
    int statusCode;
    char* statusText;
    char* taskId;
    char* messageId;
    char* textMsg;
    char* binaryData;
    int binaryDateLen;
};

typedef struct byteTunerContextCreateParameter {
    SAMICoreServerCommonParameter commonParameter;
    float shiftSecond;
    int shiftSemitone;
    float lag;
    const char* midiPath;
    const char* vocalPath;
    const char* tunedPath;
    SAMICoreServerCallbackParameter* callbackParameter;
} SAMICoreByteTunerContextCreateParameter;

enum SAMICoreWebSocketConnectionState {
    ConnectionState_ConnectUnknown = -1,
    ConnectionState_Connecting,     // 0
    ConnectionState_Disconnecting,  // 1
    ConnectionState_ConnectFailed,  // 2
    ConnectionState_ConnectClosed,  // 3
    ConnectionState_Connected,      // 4
};

struct SAMICoreWebSocketConnectionEvent {
    SAMICoreWebSocketConnectionState state;
    char* url;
    char* textMsg;
};

/*****************************/

/************** Audio Encode ****************/
typedef enum SAMI_CORE_DLL_EXPORT audioFormat {
    PCM = 0,
    WAV = 1,
    AAC = 2,
    OPUS = 3,
    OGG_OPUS = 4,
} SAMICoreAudioFormat;

struct SAMICoreAudioEncodeContextParameter {
    int sample_rate;
    int channel;
    SAMICoreAudioFormat input_format;
    SAMICoreAudioFormat output_format;
    const char* output_path;
    int bit_rate;
    int complex;
};

struct SAMICoreAudioDecodeContextParameter {
    int sample_rate;
    int channel;
    SAMICoreAudioFormat input_format;
    SAMICoreAudioFormat output_format;
    const char* output_path;
};

SAMI_CORE_EXTERN_C_END
#endif  // SAMI_CORE_DEF_H
